import React, { useRef, useEffect, useState, useContext, forwardRef, createContext, useMemo } from 'react'
import { ConfigProvider, message, Modal } from 'antd'
import Generator from 'fr-generator'
import { settings, baseSettings, globalSettings } from './config'
import widgets from '../widgets'
import { saveTableConfig, GetTableConfigJson } from '../../apis/process'
import FormRender from '../FormRender'

export const GlobalStore = createContext(null)

const FormDesigner = (props, ref) => {

  const { tableName } = props

  const { getPrefixCls } = useContext(ConfigProvider.ConfigContext)
  const prefixCls = getPrefixCls('pandaXform')
  const [visible, setVisible] = useState(false)
  const [schema, setSchema] = useState({})
  const designerRef = useRef(null)

  const settingsParent = useMemo(() => {
    let settingsParent = []
    settings.forEach(v => {
      v.widgets.forEach(s => {
        s.schema = { ...s.schema, tableNameParent: tableName }
      })
      settingsParent.push(v)
    })
    return settingsParent
  }, [settings, tableName])

  const getTableField = () => {
    console.log('setSchema', settings)
  }

  const getTableConfig = async () => {
    const { code, data } = await GetTableConfigJson({ tableName })
    if (code === 0) {
      if (typeof data === 'string') {
        setTimeout(() => {
          designerRef && designerRef.current && designerRef.current.setValue && designerRef.current.setValue(JSON.parse(data))
          setSchema(JSON.parse(data))
        }, 0)
      }
    } else {
      message.error('获取表单配置失败')
    }
  }

  useEffect(() => {
    getTableConfig()
  }, [])

  const extraButtons = [
    false,
    true,
    true,
    true,
    {
      text: '预览',
      onClick: (e) => {
        setVisible(true)
        setSchema(designerRef.current.getValue())
      }
    },
    {
      text: '提交',
      onClick: async (value) => {
        const errors = designerRef.current.getErrorFields()
        if (errors.length) {
          return message.error('请按照提示完善表单内容')
        }
        const { code, data, msg } = await saveTableConfig(designerRef.current.getValue())
        if (code === 0) {
          message.info('保存成功')
        } else {
          message.error(msg)
        }
      }
    }
  ]

  const onChange = (value) => {
    // console.log('value', value)
  }

  const onCanvasSelect = (schema) => {
    // console.log('schema', schema)
  }

  return (
    <div className={prefixCls} style={{ width: '100%', height: '100%', background: 'white' }}>
      <div style={{ height: '750px' }}>
        <Generator
          configProvider={{ prefixCls: prefixCls }}
          mapping={{
            object: 'Header',
          }}
          validation={true}
          ref={designerRef}
          extraButtons={extraButtons}
          onChange={onChange}
          widgets={widgets}
          onCanvasSelect={onCanvasSelect}
          settings={settingsParent}
          commonSettings={baseSettings}
          globalSettings={globalSettings}
        />
        <Modal
          title={tableName}
          width='80%'
          bodyStyle={{ height: '700px', overflow: 'auto' }}
          onOk={() => setVisible(false)}
          onCancel={() => setVisible(false)}
          visible={visible}
          destroyOnClose
        >
          <FormRender schema={schema} />
        </Modal>
      </div>
    </div>
  )

}

export default forwardRef(FormDesigner)